@@ -3,13 +3,14 @@
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "2.0.4"
+__version__ = "2.2.0"
 __all__ = ['Model', 'Wizard']
 from types import NoneType
 import threading
 import datetime
 from decimal import Decimal
 import proteus.config
+from proteus.pyson import PYSONDecoder
 
 _MODELS = threading.local()
 
@@ -54,6 +55,7 @@ class FieldDescriptor(object):
         super(FieldDescriptor, self).__init__()
         self.name = name
         self.definition = definition
+        self.__doc__ = definition['string']
 
     def __get__(self, instance, owner):
         if instance.id > 0 and self.name not in instance._values:
@@ -161,8 +163,7 @@ class Many2OneDescriptor(FieldDescriptor):
             value = relation(value)
         elif not value:
             value = None
-        if self.name in instance._values:
-            instance._values[self.name] = value
+        instance._values[self.name] = value
         return value
 
     def __set__(self, instance, value):
@@ -173,6 +174,10 @@ class Many2OneDescriptor(FieldDescriptor):
         super(Many2OneDescriptor, self).__set__(instance, value)
 
 
+class One2OneDescriptor(Many2OneDescriptor):
+    pass
+
+
 class One2ManyDescriptor(FieldDescriptor):
     default = []
 
@@ -218,6 +223,10 @@ class Many2OneValueDescriptor(ValueDescriptor):
         return value and value.id or False
 
 
+class One2OneValueDescriptor(Many2OneValueDescriptor):
+    pass
+
+
 class One2ManyValueDescriptor(ValueDescriptor):
     def __get__(self, instance, owner):
         value = [('add', [])]
@@ -267,6 +276,10 @@ class Many2OneEvalDescriptor(EvalDescriptor):
         return False
 
 
+class One2OneEvalDescriptor(Many2OneEvalDescriptor):
+    pass
+
+
 class One2ManyEvalDescriptor(EvalDescriptor):
     def __get__(self, instance, owner):
         return [x.id for x in getattr(instance, self.name)]
@@ -295,18 +308,21 @@ class MetaModelFactory(object):
         'many2one': Many2OneDescriptor,
         'one2many': One2ManyDescriptor,
         'many2many': Many2ManyDescriptor,
+        'one2one': One2OneDescriptor,
     }
     value_descriptors = {
         'reference': ReferenceValueDescriptor,
         'many2one': Many2OneValueDescriptor,
         'one2many': One2ManyValueDescriptor,
         'many2many': Many2ManyValueDescriptor,
+        'one2one': One2OneValueDescriptor,
     }
     eval_descriptors = {
         'reference': ReferenceEvalDescriptor,
         'many2one': Many2OneEvalDescriptor,
         'one2many': One2ManyEvalDescriptor,
         'many2many': Many2ManyEvalDescriptor,
+        'one2one': One2OneEvalDescriptor,
     }
 
     def __init__(self, model_name, config=None):
@@ -462,7 +478,7 @@ class Model(object):
     _config = None
     _fields = None
 
-    def __init__(self, id=None, _default=True, **kwargs):
+    def __init__(self, id=None, **kwargs):
         super(Model, self).__init__()
         if id:
             assert not kwargs
@@ -474,7 +490,7 @@ class Model(object):
         self._parent = None # store the parent record
         self._parent_field_name = '' # store the field name in parent record
         self._parent_name = '' # store the field name to parent record
-        if self.id < 0 and _default:
+        if self.id < 0:
             self._default_get()
 
         for field_name, value in kwargs.iteritems():
@@ -497,9 +513,6 @@ class Model(object):
     @classmethod
     def get(cls, name, config=None):
         'Get a class for the named Model'
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-
         class Spam(Model):
             __metaclass__ = MetaModelFactory(name, config=config)()
         return Spam
@@ -559,7 +572,7 @@ class Model(object):
         'Save the record'
         context = self._config.context
         if self.id < 0:
-            values = self._get_values()
+            values = self._get_values(fields=self._changed)
             self.__id = self._proxy.create(values, context)
         else:
             if not self._changed:
@@ -670,8 +683,11 @@ class Model(object):
         return res
 
     def _on_change_set(self, field, value):
-        if (self._fields[field]['type'] in ('one2many', 'many2many')
-                and not isinstance(value, (list, tuple))):
+        if self._fields[field]['type'] in ('one2many', 'many2many'):
+            if isinstance(value, (list, tuple)):
+                self._values[field] = value
+                self._changed.add(field)
+                return
             to_remove = []
             if value and value.get('remove'):
                 for record_id in value['remove']:
@@ -685,32 +701,22 @@ class Model(object):
                 for vals in value.get('add', []):
                     relation = Model.get(self._fields[field]['relation'],
                             self._config)
-                    record = relation(_default=False)
-                    for i, j in vals.iteritems():
-                        if '.' in i:
-                            continue
-                        record._values[i] = j
-                        record._changed.add(i)
                     # append without signal
-                    list.append(getattr(self, field), record)
+                    list.append(getattr(self, field), relation(**vals))
                 for vals in value.get('update', []):
                     if 'id' not in vals:
                         continue
                     for record in getattr(self, field):
                         if record.id == vals['id']:
                             for i, j in vals.iteritems():
-                                if '.' in i:
-                                    continue
                                 record._values[i] = j
                                 record._changed.add(i)
         else:
             self._values[field] = value
-        self._changed.add(field)
+            self._changed.add(field)
 
     def _on_change(self, name):
         'Call on_change for field'
-        # Import locally to not break installation
-        from proteus.pyson import PYSONDecoder
         definition = self._fields[name]
         if definition.get('on_change'):
             if isinstance(definition['on_change'], basestring):
@@ -729,6 +735,8 @@ class Model(object):
                 self._on_change_set(field, value)
             for field, value in later.iteritems():
                 self._on_change_set(field, value)
+            if self._parent:
+                self._parent._changed.add(self._parent_field_name)
         for field, definition in self._fields.iteritems():
             if not definition.get('on_change_with'):
                 continue
@@ -741,9 +749,6 @@ class Model(object):
             res = getattr(self._proxy, 'on_change_with_%s' % field)(args,
                     context)
             self._on_change_set(field, res)
-        if self._parent:
-            self._parent._changed.add(self._parent_field_name)
-            self._parent._on_change(self._parent_field_name)
 
 
 class Wizard(object):
